
import java.io.IOException;
import java.util.Locale;
import java.util.Random;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Class to read the data from a xml file from a weather station.
 * Version 20170310
 * 
 * @author F. van Slooten
 * 
 */
public class WeatherStation {

	private Document doc;
	private DocumentBuilder builder;
	public static String xml = "https://xml.buienradar.nl/"; // url we will fetch data from
	public int id; // id of the weather station we use to get weather-data from
	private NodeList station;
	private boolean connect = false;

	/**
	 * Initialize the weather station
	 */
	public WeatherStation() {
		initialize();
		selectStation(id);
	}
	
	/**
	 * Initialize a weather station with an id of a weather station
	 * @param id
	 */
	public WeatherStation(int id) {
		initialize();
		selectStation(id);
	}
	
	private void initialize() {
		builder = null;
		doc = null;
		id = 6260;
		station = null;

		// create a new DocumentBuilderFactory:
		DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();

		// Once you have a handle to the factory, you can create an instance of
		// the actual DOM parser using the following code:
		try {
			builder = dbfactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Read the xml-file with data from the site again,
	 * this will refresh the data.
	 */
	public void readData() {
		/*
		 * In order to parse a document, you call the parse method of the
		 * DocumentBuilder class. The parse method will return a Document
		 * object, which you can use to process the XML document.
		 */
		System.out.println("readData()");
		connect = false;
		try {
			doc = builder.parse(xml);
			connect = true;
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (!connect) System.out.println("Connection to "+xml+" failed.");
	}

	/**
	 * Return the temperature of the current weather station
	 * @return String with current temperature
	 */
	public String readTemperature() {
		if (!connect) {
			System.out.println("readTemperature(): There is no connection with "+xml+". A simulated value will be shown.");
			return simReadTemperature();
		}
		String t = readValue("temperatuurGC");
		if (t.compareTo("-")==0) t = "0";
		return t;
	}
	
	/**
	 * Return the name of the weather station
	 * @return String with the name of the current weather station
	 */
	public String readNameStation() {
		String name = readValue("stationnaam");
		return name.replace("Meetstation", "Station");
	}
	
	/**
	 * Read the data from the current weather station.
	 * Examples parameters: "temperatuurGC" gives the temperature,
	 * "luchtdruk" gives the air pressure.
	 * @param value
	 * @return String with the requested value
	 */
	public String readValue(String value) {
		if (!connect) {
			System.out.println("readValue(): There is no connection with "+xml+".");
			if (value.compareTo("icoonactueel")==0) {
				System.out.println("A simulated value is shown.");
				return simIcon();
			}
			System.out.println("No value is given.");
			return "";
		}
		selectStation(id);
		if (station==null) {
			System.out.println("No weather station is selected.");
		}
		else {
			for (int i = 0; i < station.getLength(); i++) {
				if (station.item(i).getNodeName().compareTo(value) == 0) {
					return station.item(i).getTextContent();
				}
			}
		}
		return "";
	}

	/**
	 * Select a station to read the data from
	 * 
	 * @param id
	 *    Number of the weather station
	 */
	public void selectStation(int id) {
		this.id = id;
		if (doc == null)
			readData();
		NodeList nlist;
		try {
			nlist = doc.getElementsByTagName("weerstation");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		if (id == 0) {
			station = nlist.item(0).getChildNodes();
			return;
		}
		for (int x = 0; x < nlist.getLength(); x++) {
			station = nlist.item(x).getChildNodes();
			for (int i = 0; i < station.getLength(); i++) {
				// check if there is a stationcode, with the parameter id
				if (station.item(i).getNodeName().compareTo("stationcode") == 0
						&& station.item(i).getTextContent().compareTo("" + id) == 0) {
					return;
				}
			}

		}
		station = null;
	}
	
	/**
	 * Generate a simulated temperature value.
	 * The temperature is generated using a Random value.
	 * 
	 * @return temperature in String
	 */
	public String simReadTemperature() {
		double start = -10.0;
		double end = 40.0;
		double random = new Random().nextDouble();
		double result = start + (random * (end - start));
		//System.out.println("simReadTemperature(): result="+result);
		return String.format(Locale.US, "%.1f", result);
	}
	
	/**
	 * Generate a simulated icon file.
	 * This is generated using a Random value.
	 * @return
	 */
	public String simIcon() {
		char[] icons = {'0','a','b','c','d','e','f','g','h','i','p'};

        // generate a random value between 0 and icons.length
		int result = new Random().nextInt(icons.length);

		//System.out.println("simIcon() result="+result);
		return xml+"icons/"+icons[result]+".gif";
	}

}
